/* Fake identd */
/* Copyright (C) 2000  Joshua Hill */
/* By Joshua Hill <josh-oss@untruth.org> */
/* License: GPL */


#include <unistd.h>
#include <syslog.h>
#include <string.h>
#include <signal.h>
#include <ctype.h>
#include <stdlib.h>

#include "fidentd.h"

/* Version .03.2 */

void getJiggyWidIt()
{
	syslog(LOG_WARNING, "Client timed out");
	_exit(-1);
}


int main(int argc, char *argv[])
{
	int retvalue;
	struct sigaction sig;
	char buffer[BUFSIZE];
	int index;
	int complete;
	char *eol;
	char serverPortStr[6], clientPortStr[6];
	int serverPort, clientPort;
	char *curPos;
	int cntr;
	char outStr[BUFSIZE];

	/* Tell syslog what to call us*/
#ifdef DEBUG
	openlog("fidentd", LOG_PERROR, LOG_DAEMON);
#else
	openlog("fidentd", 0, LOG_DAEMON);
#endif

	/* Now setup the timeout system */

	sigemptyset(&sig.sa_mask);
	sigaddset(&sig.sa_mask, SIGALRM);
	sig.sa_handler = getJiggyWidIt;
	sig.sa_flags = 0;
	if(sigaction(SIGALRM, &sig, NULL) < 0) {
		syslog(LOG_ERR, "Can't do signal handler for SIGALRM\n");
		exit(-1);
	}

	alarm(IDLE_TIME);

	/* Set some basic vars for reading the line*/
	complete = 0;
	index = 0;

	while(!complete) {
		/* Get data from the user */
		retvalue = read(STDIN_FILENO, buffer+index, (BUFSIZE - 1) - index);

		if(retvalue < 0) {
			syslog(LOG_ERR, "read for ports failed: %m");
			exit(-1);
		}

		if(retvalue == 0) {
			syslog(LOG_WARNING, "EOF reached before EOL");
			exit(-1);
		}

		index += retvalue;

		if(index > (BUFSIZE - 1)) {
			syslog(LOG_ERR, "read didn't work as advertised");
			exit(-1);
		}

		buffer[index] = 0;

		/* is there a line feed in there somewhere? */
		eol = strchr(buffer, 10);

		if(eol) {
			complete = 1;

			/* terminate the string at EOL */
			eol++;
			*eol = 0;
		} else if(index == (BUFSIZE - 1)) {
			syslog(LOG_WARNING, "client wrote BUFSIZE characters without EOL");
			exit(-1);
		}

	}

	/* Hey, we got what looks like a complete line */
	/* Now let's parse it*/
	curPos = buffer;

	/* Get rid of leading whitespace */
	while((*curPos) && (isspace(*curPos)))
		curPos++;

	/*It has to be a number at this point*/
	if(!isdigit(*curPos)) {
		syslog(LOG_WARNING, "Invalid query format (server port)");
		exit(-1);
	}

	/* Now get the server port number.  Can't be larger than 65536 */
	cntr=0;
	while((*curPos) && (isdigit(*curPos))) {
		if(cntr < 5) {
			serverPortStr[cntr] = *curPos;
			cntr++;
			curPos++;
		} else {
			syslog(LOG_WARNING, "Invalid Server Port number");
			exit(-1);
		}
	}

	serverPortStr[cntr] = 0;
	serverPort = atoi(serverPortStr);

	/* Trailing whitespaces */
	while((*curPos) && (isspace(*curPos)))
		curPos++;

	/* You have to have a comma in there somewhere */	
	if(*curPos == ',') {
		curPos++;
	} else {
		syslog(LOG_WARNING, "Invalid query format (separator)");
		exit(-1);
	}

	/* More leading whitespaces */
	while((*curPos) && (isspace(*curPos)))
		curPos++;

	/*It has to be a number at this point*/
	if(!isdigit(*curPos)) {
		syslog(LOG_WARNING, "Invalid query format (client port)");
		exit(-1);
	}

	/* Now get the client port number.  Can't be larger than 65536 */
	cntr=0;
	while((*curPos) && (isdigit(*curPos))) {
		if(cntr < 5) {
			clientPortStr[cntr] = *curPos;
			cntr++;
			curPos++;
		} else {
			syslog(LOG_WARNING, "Invalid Client Port number");
			exit(-1);
		}
	}

	clientPortStr[cntr] = 0;
	clientPort = atoi(clientPortStr);

	/* Don't really care about the rest of the string */

	strcpy(outStr, serverPortStr);
	strcat(outStr, " , ");
	strcat(outStr, clientPortStr);

	if((serverPort < 1) || (serverPort > 65535)) {
		strcat(outStr, PORT_ERROR);
	} else if((clientPort < 1) || (clientPort > 65535)) {
		strcat(outStr, PORT_ERROR);
	} else {
		strcat(outStr, USER_RESP);
	}

	retvalue = write(STDOUT_FILENO, outStr, strlen(outStr));

	if(retvalue < 0) {
		syslog(LOG_ERR, "Error writing to client: %m");
		exit(-1);
	}

	if(retvalue != strlen(outStr)) {
		syslog(LOG_WARNING, "Short writing to client: %m");
		exit(-1);
	}

	exit(0);
}
